"""DILL - Cross-database interface layer compatible with MySQL, ODBC, Gadfly.

For more information view the README.txt, USERGUIDE.txt, and DEVELOPERGUIDE.txt files in the module directory or
visit http://lemon.xecos.com/docs/ for HTML versions of the same files.

Points to note:

    - Results aren't automatically committed.
    - You can alter the fetch and execute modes.

    - Seconds must be whole seconds.

    - Should be able to call the ColTypes table something else and block the use of the name ColTypes.
    - Important to commit() changes before and after creating/altering/dropping code.
"""
__docformat__ = "restructuredtext en"
__author__ = "James Gardner [email: james at xecos.com]"
__date__ = "02 July 2003"
__version__ = "Version 0.1 for LEMON 0.2"[-3:]
__credits__ = "Guido van Rossum, for an excellent programming language."

import lemon, drivers

from lemon.date import *
from errors import DateError, DBError
   
def connect(type, database, dir='', startup='', user='', password='', host='', socket='', port=''):
    
    """Connects to the appropriate database and returns a lemon.db cursor (which is derived from a normal cursor).
    Options available are: type, syntax, user, password, protocol, host, socket, port, database
    
    Specifying no options or setting defaults=True loads the default settings.
    """
    if type == 'mysql':
        if dir:
            raise DBError("The MySQL driver does not take the parameter 'dir'."%dir)
        else:
            import drivers.mysql
            return drivers.mysql.mysqlConnection(database, user, password, host, socket, port)
    elif type == 'odbc':
        if dir:
            raise DBError("The ODBC driver does not take the parameter 'dir'.")
        if startup:
            raise DBError("The ODBC driver does not take the parameter 'startup'.")
        if user:
            raise DBError("The ODBC driver does not take the parameter 'user'.")
        if password:
            raise DBError("The ODBC driver does not take the parameter 'password'.")
        if host:
            raise DBError("The ODBC driver does not take the parameter 'host'.")
        if socket:
            raise DBError("The ODBC driver does not take the parameter 'socket'.")
        if port:
            raise DBError("The ODBC driver does not take the parameter 'port'.")
        else:
            import drivers.odbc
            return drivers.odbc.odbcConnection(database)
    elif type == 'gadfly':
        if user:
            raise DBError("The Gadfly driver does not take the parameter 'user'.")
        if password:
            raise DBError("The Gadfly driver does not take the parameter 'password'.")
        if host:
            raise DBError("The Gadfly driver does not take the parameter 'host'.")
        if socket:
            raise DBError("The Gadfly driver does not take the parameter 'socket'.")
        if port:
            raise DBError("The Gadfly driver does not take the parameter 'port'.")
        else:
            import drivers.gc
            if startup == True:
                return drivers.gc.gadflyStartupConnection(database, dir)
            elif not startup:
                return drivers.gc.gadflyConnection(database, dir)
            else:
                raise DBError("The parameter 'startup' should be True or False, not '%s'."%startup)
    else:
        raise DBError("There is no such database type '%s'."%type)

#~ def createGadflyDatabase(name, directory):
    #~ "Setup a gadfly database named 'name' in the directory 'directory' and add the ColTypes table."
    #~ import lemon.db.drivers.gc
    #~ import gadfly
    #~ import os.path
    #~ connection = gadfly.gadfly()
    #~ connection.startup(name, os.path.abspath(directory))
    #~ database = lemon.db.drivers.gc.gadflyCursor(conn=connection)
    #~ database.createColTypesTable()
    #~ database.commit()
    #~ database.close()


    #~ allowedParams = ['type','dir','user','password','host','socket','port','database','defaults']
    #~ if (dsn.has_key('defaults') and dsn['defaults'] == True) or len(dsn) == 0:
        #~ defaultDsn =    {   
                            #~ 'type':lemon.config.get('db','type'),
                            #~ 'dir':lemon.config.get('db','gadfly.dir'),
                            #~ 'user':lemon.config.get('db','user'),
                            #~ 'password':lemon.config.get('db','password'),
                            #~ 'host':lemon.config.get('db','host'),
                            #~ 'socket':lemon.config.get('db','socket'),
                            #~ 'port':lemon.config.get('db','port'),
                            #~ 'database':lemon.config.get('db','database'),
                        #~ }
        #~ dsn = lemon.addToDict(defaultDsn, dsn)
        #~ olddsn =  lemon.addToDict(dsn, {                           
                                            #~ 'autoExecute':True,
                                            #~ 'fetchMode':'tuple',
                                            #~ 'autoConvert':'table',
                                            #~ 'types':{},
                                        #~ }
                                    #~ )
        #~ dsn = lemon.keepInDict(dsn, allowedParams)
        #~ #if not fulldsn == dsn:
        #~ #    for k in dsn.keys():
        #~ #        if not fulldsn.has_key(k):
        #~ #            raise DBError("The parameter '%s' is not a valid parameter for a database connection."%(k))
        #~ #dsn = fulldsn
    #~ # Setup the DB Specific Functions:
    #~ else:
        #~ olddsn =  lemon.addToDict(dsn, { 'autoExecute':True,
                                            #~ 'fetchMode':'tuple',
                                            #~ 'autoConvert':'table',
                                            #~ 'types':{},
                        #~ }, False)
        #~ dsn = lemon.keepInDict(dsn, allowedParams)