"""DBBuilder - Typed SQL Abstraction Layer.

Still to do:        
        Forms Code -> pass the form object a Row class.
        (Transaction Support.) In the documentation.
            ##### Problem with default values in defaults???
            
        Driver/Interface model for implementing anydb, other drivers etc...
   
BUGS 
    - Dates don't work properly from the table(rowid).dict() method.
    - Should be able to choose to use the Lemon date classes with a function.

Developer Documentation:

    Rough guide in case I forget!!
    
    Table Class
    
    User constructs a class derived from table -> In the initialisation of the Table base class a list of column
    objects is created based on the names and types of the columns specified in the derived class. Any custom
    methods are applied to the objects by assigning the methods to currently existing methods. The column names
    are error checked
    
    Row Class
    
    The row class contains the values from one row of the table. It holds the data as a list of column values
    the same as in the table class. It has a load of overloaded methods to allow dictionary access etc.
    
    We now have:
    
    row.middleInitial           #  - The value of the column middleInitial for the row row.
    row.var.middleInitial       #  - A Query Builder object for middleInitial to create WHERE clauses.
    row.column.middleInitial    #  - The middleInitial form field, containing the column value middleInitial for
                                #    use as a form field.
                                
    row.column.middleInitial can be used in an html.form.Form setup() function exactly as if it was an html.field
    object. This means that you can easily create correctly validating html forms which makes life much much easier
    if you are wrting a web application as it means you don't have to do any work yourself! Just create your own
    middleInitial_valid() funciton in the class constructor, setting any errors and this will be used to validate
    your field.
    
    ColType Class
    
    All column type classes are dervided from ColType and have a number of different methods to be overloaded in
    the creation of new column types. Each ColType object contains the data from one table field for one row as
    well as containing methods to validate and manipulate that value. The validation methods etc, are used from 
    all over the module to make sure values are correct.
    
    ColType classes are effectively a derivitive of both an html field and an imaginary db field. The class
    achieves this by having methods, identical to the html field whilst also implementing its own methods.
    
    An ideal solution would be to have one class Type that acted both as a db field, an html field and a data
    type. We cannot do things this way though because we want data types such as strings to be used and we 
    cannot add our own methods to a string. So to save having to use classes for string and int we do it this
    complicated way instead!
    
    DataType Classes
    
    These are classes used to store the data for a particular ColType field. For example Date stores information
    for DateCol classes. DataType classes should have the __cmp__() method defined for comparison and should do
    bsaic error checking for the data type in the constructor. ColType fields should only take the corresponding
    DataType object as the value of the field and the validation methods should be written with this in respect.
    The exceptions are the string and number based classes already written which take the corresponding Python
    type as the data object.
    
    Query Builder
    
    This class holds the names of all the columns. A QueryBuilder object is obtained from the Table().query. The
    class allows attribute access to columns returning a Query class for that column.
    
    Query Class
    
    This is a class with a load of overloaded operators to allow the construction of SQL queries in natural python
    syntax. Values are stored in a list, SQL is stored as the name. As the expression is built, '%s's are placed
    where the SQL values should be and the new SQL is passed as the name of the next Query object which is
    returned. In this way the query gets longer as does the attribute list. When the __str__() method is called
    the values are all error checked against their type using the ColType.checkType() method and if successful
    encoded using ColType.encodeAsQuery() and full SQL query is returned as SQL for use in the WHERE clause.
    
    Column Class
    
    This is simply a class to act like a list but be able to access ColType objects by name via an attribute
    access system so that adding them to a form looks pretty in code.
    
    

    
    #Mappings

    Foreign Key and Mapping implementations are just a matter of doing a seond query.
    Maybe map the result to a mapping object accessable as an array.
    
    Deleting an object with a foreighn key deletes them too.. so do something with them first!
    
    Implement slicing, contains for rows and mapping objects.
    

All datatypes must implment their own validation and the __str__ method to convert them to SQL.
They must also implemnt the comparison operaotrs, as must the rows column.

They should all have the .sql() method.

Constructing Objects/Data Types
-------------------------------

If the object is not an integer it is assumed that str(object) will generate the SQL together with necessary
quoting.


Still to implement:

    (Timestamp), Pickle???
    Auto generation of currentCols
    Code for unique.
    rename name() etc to sql_name()
    Simple structure for inheriting
    Re-implemnt col-override.
    
Adding a field:
    - Derive from ColType
    - Add name to currentCols
    - Add to Query.checkTypes() method for validation.
    - Add a new DateType to hold the value.
    
Warnings:
    - Uniqueness checking is done by the database. This option just sets the UNIQUE when creating
      the table so make sure your table has it specified so that it works!
      
      
QueryBuilder - Code to generate SQL for the WHERE statements.

Still To Do:
    Error check the Func arguments.
    
# XXX Remove autoincrement.. do it manually.

Note.. implemented asList() for Column class as a hack rather than writing an iterator.
"""

#################################################################################################################
#
# Constants
#

#Reserved SQL keywords and other reserved words.
reservedWords = [
                'ID', # Used by the Table class for the primary key column.
                'ADD',
                'ALL',
                'ALTER',
                'AND',
                'ANY',
                'AS',
                'ASC',
                'AUTOINCREMENT',
                'AVA',
                'BETWEEN',
                'BINARY',
                'BIT',
                'BOOLEAN',
                'BY',
                'CREATE',
                'BYTE',
                'CHAR',
                'CHARACTER',
                'COLUMN',
                'CONSTRAINT',
                'COUNT',
                'COUNTER',
                'CURRENCY',
                'DATABASE',
                'DATE',
                'DATETIME',
                'DELETE',
                'DESC',
                'DISALLOW',
                'DISTINCT',
                'DISTINCTROW',
                'DOUBLE',
                'DROP',
                'EXISTS',
                'FROM',
                'FLOAT',
                'FLOAT4',
                'FLOAT8',
                'FOREIGN',
                'GENERAL',
                'GROUP',
                'GUID',
                'HAVING',
                'INNER',
                'INSERT',
                'IGNORE',
                'IMP',
                'IN',
                'INDEX',
                'INT',
                'INTEGER',
                'INTEGER1',
                'INTEGER2',
                'INTEGER4',
                'INTO',
                'IS',
                'JOIN',
                'KEY',
                'LEFT',
                'LEVEL',
                'LIKE',
                'LOGICAL',
                'LONG',
                'LONGBINARY',
                'LONGTEXT',
                'MAX',
                'MEMO',
                'MIN',
                'MOD',
                'MONEY',
                'NOT',
                'NULL',
                'NUMBER',
                'NUMERIC',
                'OLEOBJECT',
                'ON',
                'PIVOT',
                'OPTION',
                'PRIMARY',
                'ORDER',
                'OUTER',
                'OWNERACCESS',
                'PARAMETERS',
                'PERCENT',
                'REAL',
                'REFERENCES',
                'RIGHT',
                'SELECT',
                'SET',
                'SHORT',
                'SINGLE',
                'SMALLINT',
                'SOME',
                'STDEV',
                'STDEVP',
                'STRING',
                'SUM',
                'TABLE',
                'TABLEID',
                'TEXT',
                'TIME',
                'TIMESTAMP',
                'TOP',
                'TRANSFORM',
                'UNION',
                'UNIQUE',
                'UPDATE',
                'VALUE',
                'VALUES',
                'VAR',
                'VARBINARY',
                'VARCHAR',
                'VARP',
                'WHERE',
                'WITH',
                'YESNO',
            ]


#################################################################################################################
#
# Imports
#

# import from lemon
import lemon.html.form
import lemon.db # Timestamp, DateFromTicks, TimeFromTicks, TimestampFromTicks Not needed

# import from standard modules
from inspect import getmembers, ismethod, isclass
from types import InstanceType

# import our driver
import drivers.db as driver


#################################################################################################################
#
# Errors
#

# import from db.builder NB.. these could all be in one file if you liked!
class SQLError(Exception):
    pass
    
#################################################################################################################
#
# Columns
#



class SQLCol:
    
    def __init__(self, notNULL=False, unique=False, default=None, description=None, treatEmptyStringAsNull=True, hidden=False):
        #, database=None, table=None, name=None, description='', error='', default=None, value=None): # XXX Not done yet.
                
        self.sql_notNULL                = notNULL
        self.sql_hidden                 = hidden
        self.sql_unique                 = unique
        self.sql_default                = default
        self.sql_description            = description
        self.sql_treatEmptyStringAsNull = treatEmptyStringAsNull   
        
        # Private variables
        self.sql_error                  = None
        self.sql_classID                = "THIS IS A James Gardner ColType Derived Class." # XXX Cheap Hack!
        self.sql_initialised            = False
        
        # Variables to initialise later
        self.sql_table                  = None
        self.sql_value                  = None   #Decoded Value
        self.sql_database               = None
        self.sql_name                   = None  
        #self.sql_field                  = "hi"
        

    def _init_(self, name, table, database):
        self.sql_initialised = True
        self.sql_setName(name)
        self.sql_setTable(table)
        self.sql_database=database
        
    def __repr__(self):
        return ("<SQLCol, type=%s, name=%s, value=%s>"%(self.__class__.__name__, self.sql_getName(), self.sql_value)).replace('<','&lt;').replace('>','&gt;')
    
    # XXX def __str__(self):
    #    return "<SQLCol, type=%s, name=%s, value=%s>"%(self.__class__.__name__, self.sql_getName(), self.sql_value)
        
    #
    # Internal Get/Set Methods
    #

    def sql_setNotNULL(self, t):
        self.sql_notNULL = t
    def sql_getNotNULL(self):
        return self.sql_notNULL
        
    def sql_setTable(self, t):
        self.sql_table = t
    def sql_getTable(self):
        return self.sql_table
        
    def sql_setError(self, error):
        self.sql_error = error
    def sql_getError(self):
        return self.sql_error

    def sql_getDescription(self):
        return self.sql_description
    def sql_setDescription(self, d):
        self.sql_description = d

    def sql_setName(self, name):
        self.sql_name = name
        if not self.sql_getDescription():
            self.sql_setDescription(name)
    def sql_getName(self):
        return self.sql_name
        
    def sql_getValue(self):
        if self.sql_initialised:
            return self.sql_value
            #~ if self.__class__.__name__ == 'ForeignKey':
                #~ raise SQLError('Foreign Key not implemented.')
            #~ else:
        else:
            raise SQLError("Column not initialised.")
            
        
    def sql_setValue(self, value):
        self.sql_value = value
        self.sql_setField(self.sql_value)
        
    #
    # Internal Funcitons
    #
    
    def sql_valid(self, value):
        "Generic Error Checking on the values."
        if self.sql_notNULL == True:
            if value == None:
                self.sql_setError("This is a required field.")
                return False
            elif type(value) == type(''):
                if self.sql_treatEmptyStringAsNull:
                    if value == '':
                        self.sql_setError("This is a required field.")
                        return False
            elif type(value) == type([]):
                if lemon.allNone(value):
                    self.sql_setError("This is a required field.")
                    return False
        # XXX Checking unique here is left to the database for efficiency. 
        # XXX Unless gadfly!!!
        
        #if self.sql_database.__class__.__name__ == 'gadflyCursor':
        #    raise TypeError(self.sql_database.__class__.__name__)
            
        return True

    def sql_setDB(self, value):

        if self.sql_initialised:
            # XXX No corresponding getDB.. get always from self.sql_value
            #~ if self.__class__.__name__ == 'MultipleJoin':
                #~ raise SQLError('You cannot set the value of a ForeignKey. Insetead, set the fields directly.')
            #~ else:
            if self.sql_valid(value):
                self.onSet()            # Call the custom onSet() function first.
                self.sql_value = value      # So the get method doesn't need to be called again.
                value = self.sql_encode(value)
                driver.updateRow(self.sql_getTable(),self.sql_getName(), value)
                #self.sql_database.commit()
                self.sql_setField(self.sql_value)
            else:
                raise SQLError(self.sql_error)
        else:
            raise SQLError("Column not initialised.")

    #
    # Methods to override
    #

    def sql_encodeAsQuery(self, value):
        if type(value) == type(''):
            stri = value.replace("'","''")
            return "'"+stri+"'"
        elif type(value) in [type(1),type(1L), type(1.001)]:
            return str(value)
        elif type(value) == InstanceType:
            if value.__class__.__name__ in ['Date','Time','DateTime']:
                return "'"+str(value)+"'"
            else:
                raise SQLError("The value '%s' doesn't seem valid...")
            
        
    def sql_encode(self, value):
        return value
        
    def sql_decode(self, value):
        return value

    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        raise SQLError('This function should be overridden in the derived class.')
    
    #~ #
    #~ # Overriden in the Table definition
    #~ #

    #~ def onSet(self):
        #~ "Over-ridden"
        #~ pass
        
    #~ def onGet(self, value): #value being retrieved
        #~ "Over-ridden"
        #~ pass
    
    #~ def isValid(self, value):
        #~ "Over-ridden"
        #~ return True


class ColType(SQLCol):
    
    def __init__(self, notNULL=False, unique=False, default=None, description=None, treatEmptyStringAsNull=True, hidden=False):
        SQLCol.__init__(self, notNULL=notNULL, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull, hidden=hidden)
    
    #
    # Internal Methods
    #
 
    def sql_valid(self, value):
        "Generic Error Checking on the values."
        if not SQLCol.sql_valid(self, value):
            self.sql_field.setError(self.sql_getError())
            return False
        return True
        
    def sql_isUnique(self, value):
        if self.sql_unique == True:               
            return driver.colUnique(self.sql_getTable(), self.sql_getName(), value, self.sql_type())
        return True

    
    #
    # HTML Functions
    #

    def populate(self, form):
        self.sql_field.populate(form)

    def html(self):
        "Should be overridden."
        # The valid method sets the error in this class so we need to transfer it!
        return self.sql_field.html()
        
    def frozen(self):
        "Should be overridden."
        return self.sql_field.frozen()

    def hidden(self):
        "Should be overridden."
        return self.sql_field.hidden()
               
    def get(self):
        "Perform any final changes on the finished value then return."
        return self.sql_field.get()
        
    def builderValue(self):
        # WARNING: This gets the value from the form field.. it does nothing to the database.
        return self.sql_field.builderValue()
        
    def name(self):
        return self.sql_getName()
        
    def type(self):
        return self.sql_field.type()
        
    def error(self):
        if self.sql_error:
            return str(self.sql_error)
        if self.sql_field.error():
            return str(self.sql_field.error())
        return ''
               
    def description(self):
        return self.sql_description
                        
    def valid(self, value):
        "Field level validation."
        # Try to validate the form using its own constraints first.. ours will be more taxing and less rich in 
        # error information.
        if self.sql_field.valid(value):
            if self.sql_valid(value):
                if self.sql_isUnique(value): # XXX value or self.builderValue() ? 
                    return True
                else:
                    self.sql_field.setError('Values for this field must be unique but a field "%s" alrady exists. Please choose a different value.'%value)
                    return False
            else:
                self.sql_field.setError(self.sql_getError())
                return False
        else:
            return False

class StringCol(ColType):
    
    def sql_encodeAsQuery(self, value):
        stri = value.replace("'","''")
        return "'"+stri+"'"
                
    def sql_type(self):
        return "STRING"

    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        value = str(value)
        if value == 'None':
            value = ''
        self.sql_field = self.sql_buildField(self.sql_getName(), value, self.sql_getDescription())
        
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Input(name, default=value, description=description, maxlength=255)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if not ColType.sql_valid(self, value):
            return False
        else:
            validates = True
            if type(value) <> type(''):
                self.sql_setError('StringCol values should be strings not %s'%type(value))
                return False
            elif len(value) > 255:
                self.sql_setError('StringCol values should be less than 255 chracters long.')
                return False
            return validates
        
    def sql_encodeAsQuery(self, value):
        stri = value.replace("'","''")
        return "'"+stri+"'"

# XXX TODO: Update the options to accept only pairs of strings.. do proper error checking on the values.

class StringSelectCol(StringCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        StringCol.__init__(self, notNULL=False, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = options
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Select(name, options=self.sql_options, default=value, description=description)

class StringRadioGroupCol(StringCol):
    
    def __init__(self,  unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        StringCol.__init__(self, notNULL=True, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = options
            
    def sql_buildField(self, name, value, description):
        return lemon.html.form.RadioGroup(name, options=self.sql_options, default=value, description=description)


class CharCol(StringCol):

    def sql_type(self):
        return "CHAR"
        
    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        
        if value == None:
            value = ''
        else:
            value = str(value)
        self.sql_field = self.sql_buildField(self.sql_getName(), value, self.sql_getDescription())
        
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Input(name, default=value, description=description, size=1, maxlength=1)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if not StringCol.sql_valid(self, value):
            return False
        else:
            validates = True
            if type(value) <> type(''):
                self.sql_setError('Values should be strings not %s'%type(value))
                return False
            elif len(value) > 1:
                self.sql_setError('Value should be 1 character long not %s.'%len(value))
                return False
            return validates

    def sql_encodeAsQuery(self, value):
        stri = value.replace("'","''")
        return "'"+stri+"'"

class CharSelectCol(CharCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        CharCol.__init__(self, notNULL=False, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = options
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Select(name, options=self.sql_options, default=value, description=description)

class CharRadioGroupCol(CharCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        CharCol.__init__(self, notNULL=True, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = options
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.RadioGroup(name, options=self.sql_options, default=value, description=description)


class TextCol(StringCol):  # XXX Got the inheritance order the wrong way around.
    
    def __init__(self, notNULL=False, unique=None, default=None, description=None, treatEmptyStringAsNull=True):
        if unique <> None:
            raise SQLError("TextCol fields cannot be specified as unique.")
        StringCol.__init__(self, notNULL=notNULL, unique=False, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)

    def sql_encodeAsQuery(self, value):
        stri = value.replace("'","''")
        return "'"+stri+"'"
                
    def sql_type(self):
        return "BINARY"
        
    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        value = str(value)
        if value == 'None':
            value = ''
        self.sql_field = lemon.html.form.TextArea(self.sql_getName(), default=value, description=self.sql_description, cols="40", rows="6")
        
    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if not ColType.sql_valid(self, value):
            return False
        else:
            validates = True
            if type(value) <> type(''):
                self.sql_setError('Values should be strings not %s'%type(value))
                return False
            return validates
    
    def sql_encodeAsQuery(self, value):
        stri = value.replace("'","''")
        return "'"+stri+"'"
        
class IntegerCol(ColType):
    
    def sql_encodeAsQuery(self, value):
        return str(value)
        
    def sql_type(self):
        return "INTEGER"
        
    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        value = str(value)
        if value == 'None':
            value = ''
        self.sql_field = self.sql_buildField(self.sql_getName(), value, self.sql_getDescription())
        
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Input(self.sql_getName(), default=value, description=self.sql_description, maxlength=11)

    def builderValue(self):
        # WARNING: This gets the value from the form field.. it does nothing to the database.

        value = self.sql_field.builderValue()
        if value == '' or value == None:
            return None
        else:
            return int(value)

    def valid(self, value):
        "Field level validation."
        if self.sql_field.valid(value):
            if value==None or value=='':
                if self.sql_notNULL == True:
                    self.sql_field.setError("This is a required field.")
                    return False
                else:
                    return True
            else:
                try:
                    value = int(value)
                except:
                    self.sql_field.setError("Not a valid integer.")
                    return False
                else:
                    if self.sql_valid(value):
                        if not self.sql_isUnique(self.builderValue()):
                            self.sql_field.setError('Values for this field must be unique but a field "%s" alrady exists. Please choose a different value.'%value)
                            return False
                        else:
                            return True
                    else:
                        self.sql_field.setError(self.sql_getError())
                        return False
        else:
            return False
            
    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        
        if not ColType.sql_valid(self, value):
            return False
        else:
            if value==None:
                return True
            else:
                validates = True
                if type(value) not in [type(1),type(1L),type(1.001)]:
                    self.sql_setError('IntegerCol values should be integers not %s'%type(value))
                    return False
                elif value > 2147483647:
                    self.sql_setError('IntegerCol values should be less than or equal to 2147483647.')
                    return False
                elif value < -2147483647:
                    self.sql_setError('IntegerCol values should be greater than or equal to -2147483648.')
                    return False
                return validates
        

    def sql_encodeAsQuery(self, value):
        return value
        
class IntegerSelectCol(IntegerCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        IntegerCol.__init__(self, notNULL=False, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = options
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Select(name, options=self.sql_options, default=value, description=description)

class IntegerRadioGroupCol(IntegerCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        IntegerCol.__init__(self, notNULL=True, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = options
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.RadioGroup(name, options=self.sql_options, default=value, description=description)



class FloatCol(IntegerCol):
    
    def sql_encodeAsQuery(self, value):
        return str(value)
        
    def sql_type(self):
        return "FLOAT"

    def valid(self, value):
        "Field level validation."
        if self.sql_field.valid(value):
            if value==None or value=='':
                if self.sql_notNULL == True:
                    self.sql_field.setError("This is a required field.")
                    return False
                else:
                    return True
            else:
                try:
                    value = float(value)
                except:
                    self.sql_field.setError("Not a valid number.")
                    return False
                else:
                    if self.sql_valid(value):
                        if not self.sql_isUnique(self.builderValue()):
                            self.sql_field.setError('Values for this field must be unique but a field "%s" alrady exists. Please choose a different value.'%value)
                            return False
                        else:
                            return True
                    else:
                        self.sql_field.setError(self.sql_getError())
                        return False
        else:
            return False

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if not ColType.sql_valid(self, value):
            return False
        else:
            if value==None:
                return True
            else:
                validates = True
                if type(value) <> type(None):
                    if type(value) not in [type(1.0002), type(1L), type(1)]:
                        self.sql_setError('FloatCol values should be type Float, Int or Long, not %s'%type(value))
                        return False
                return validates
        
    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        value = str(value)
        if value == 'None':
            value = ''
        self.sql_field = self.sql_buildField(self.sql_getName(), value, self.sql_getDescription())
        
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Input(self.sql_getName(), default=value, description=self.sql_description, maxlength=255)
        
    def builderValue(self):
        value = self.sql_field.builderValue()
        if value == '' or value == None:
            return None
        else:
            return float(value)

class FloatSelectCol(FloatCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        FloatCol.__init__(self, notNULL=False, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = options
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Select(name, options=self.sql_options, default=value, description=description)

class FloatRadioGroupCol(FloatCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        IntegerCol.__init__(self, notNULL=True, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = options
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.RadioGroup(name, options=self.sql_options, default=value, description=description)

class DateTimeCol(ColType):
    
    def sql_encodeAsQuery(self, value):
        return "'"+value.asSQL()+"'"
 
    def sql_type(self):
        return "DATETIME"

    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            if lemon.allNone(value):
                return None
            else:
                lemon.date.datetime(int(value[0:4]),int(value[5:7]),int(value[8:10]),int(value[11:13]),int(value[14:16]),int(value[17:19]))
        else:
            raise SQLError("'%s' is not a valid Time."%value)
            
    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        self.sql_field = lemon.html.form.DateTime(self.sql_getName(), default=value, description=self.sql_description)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        
        if not ColType.sql_valid(self, value):
            return False
        else:
            validates = True
            if type(value) is type([]):
                if lemon.allNone(value):
                    return True # Value is None and that is allowed!
                else:
                    try:
                        lemon.date.datetime(int(value[0:4]),int(value[5:7]),int(value[8:10]),int(value[11:13]),int(value[14:16]),int(value[17:19]))
                    except lemon.db.DBError,e:
                        self.sql_setError(str(e))
                        return False
                    except ValueError:
                        self.sql_setError("Please choose a valid date and time.")
                        return False
                # XXX Assume it is a DateTime object.
            return validates


class DateTimeSelectCol(StringCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        StringCol.__init__(self, notNULL=False, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = []
            for option in options:
                self.sql_options.append((option.asSQL(), option.asSQL()))
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Select(name, options=self.sql_options, default=value, description=description)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if type(value) == type(''):
            if not StringCol.sql_valid(self, value):
                return False
            else:
                try:
                    lemon.db.DateTime(sql=value)
                except lemon.db.DateError:
                    self.sql_setError('The date chosen is not valid.')
                    return False
                else:
                    return True
        else:
            if value.__class__.__name__ == 'DateTime':
                return True
            return False
    
    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            return lemon.db.DateTime(sql=value)
        else:
            raise SQLError("'%s' is not a valid DateTime."%value)
            
class DateTimeRadioGroupCol(StringCol):
    
    def __init__(self,  unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        StringCol.__init__(self, notNULL=True, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = []
            for option in options:
                self.sql_options.append((option.asSQL(), option.asSQL()))
            
    def sql_buildField(self, name, value, description):
        return lemon.html.form.RadioGroup(name, options=self.sql_options, default=value, description=description)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if type(value) == type(''):
            if not StringCol.sql_valid(self, value):
                return False
            else:
                try:
                    lemon.db.DateTime(sql=value)
                except lemon.db.DateError:
                    self.sql_setError('The date chosen is not valid.')
                    return False
                else:
                    return True
        else:
            if value.__class__.__name__ == 'DateTime':
                return True
            return False
    
    
    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            return lemon.db.DateTime(sql=value)
        else:
            raise SQLError("'%s' is not a valid DateTime."%value)
            
            
class DateCol(ColType):
    
    def sql_encodeAsQuery(self, value):
        return "'"+value.asSQL()+"'"
        
    def sql_type(self):
        return "DATE"
        
    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            if lemon.allNone(value):
                return None
            else:
                return lemon.date.date(int(value[0:4]),int(value[5:7]),int(value[8:10]))
        else:
            raise SQLError("'%s' is not a valid Date."%value)
            
    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        self.sql_field = lemon.html.form.Date(self.sql_getName(), default=value, description=self.sql_description)
        
    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        
        if not ColType.sql_valid(self, value):
            return False
        else:
            validates = True
            if type(value) is type([]):
                if lemon.allNone(value):
                    return True # Value is None and that is allowed!
                else:
                    try:
                        lemon.date.date(int(value[0:4]),int(value[5:7]),int(value[8:10]))
                    except lemon.db.DBError,e:
                        self.sql_setError(str(e))
                        return False
                    except ValueError:
                        self.sql_setError("Please choose a valid date.")
                        return False
            # XXX Assume it is a date object.
            return validates
    
class DateSelectCol(StringCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        StringCol.__init__(self, notNULL=False, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = []
            for option in options:
                self.sql_options.append((option.asSQL(), option.asSQL()))
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Select(name, options=self.sql_options, default=value, description=description)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if type(value) == type(''):
            if not StringCol.sql_valid(self, value):
                return False
            else:
                try:
                    lemon.db.Date(sql=value)
                except lemon.db.DateError:
                    self.sql_setError('The date chosen is not valid.')
                    return False
                else:
                    return True
        else:
            if value.__class__.__name__ == 'Date':
                return True
            return False
    
    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            return lemon.db.Date(sql=value)
        else:
            raise SQLError("'%s' is not a valid Date."%value)
            
class DateRadioGroupCol(StringCol):
    
    def __init__(self,  unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        StringCol.__init__(self, notNULL=True, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = []
            for option in options:
                self.sql_options.append((option.asSQL(), option.asSQL()))
            
    def sql_buildField(self, name, value, description):
        return lemon.html.form.RadioGroup(name, options=self.sql_options, default=value, description=description)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        # XXX Check if it is an SQL or Date object. etc.....
        if type(value) == type(''):
            if not StringCol.sql_valid(self, value):
                return False
            else:
                try:
                    lemon.db.Date(sql=value)
                except lemon.db.DateError:
                    self.sql_setError('The date chosen is not valid.')
                    return False
                else:
                    return True
        else:
            if value.__class__.__name__ == 'Date':
                return True
            return False
    
    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            return lemon.db.Date(sql=value)
        else:
            raise SQLError("'%s' is not a valid Date."%value)
            
class TimeCol(ColType):
    
    def sql_encodeAsQuery(self, value):
        return "'"+value.asSQL()+"'"
        
    def sql_type(self):
        return "TIME"
        
    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            if lemon.allNone(value):
                return None
            else:
                return lemon.date.time(int(value[0:2]),int(value[3:5]))
        else:
            raise SQLError("'%s' is not a valid Time."%value)

    def sql_setField(self, value):
        "Should be set to something like self.sql_field = lemon.html.field.Input(...)."
        self.sql_field = lemon.html.form.Time(self.sql_getName(), default=value, description=self.sql_description)
        
    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        
        if not ColType.sql_valid(self, value):
            return False
        else:
            validates = True
            if type(value) is type([]):
                if lemon.allNone(value):
                    return True # Value is None and that is allowed!
                else:
                    validates = True
                    if type(value) is type([]):
                        try:
                            lemon.date.time(int(value[0:2]),int(value[3:5]),int(value[6:8]))
                        except lemon.db.DBError,e:
                            self.sql_setError(str(e))
                            return False
                        except ValueError:
                            self.sql_setError("Please choose a valid time.")
                            return False
                    # XXX Assume it is a date object.
            return validates

class TimeSelectCol(StringCol):
    
    def __init__(self, unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        StringCol.__init__(self, notNULL=False, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = []
            for option in options:
                self.sql_options.append((option.asSQL(), option.asSQL()))
    
    def sql_buildField(self, name, value, description):
        return lemon.html.form.Select(name, options=self.sql_options, default=value, description=description)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if type(value) == type(''):
            if not StringCol.sql_valid(self, value):
                return False
            else:
                try:
                    lemon.db.Time(sql=value)
                except lemon.db.DateError:
                    self.sql_setError('The date chosen is not valid.')
                    return False
                else:
                    return True
        else:
            if value.__class__.__name__ == 'Time':
                return True
            return False
    
    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            return lemon.db.Time(sql=value)
        else:
            raise SQLError("'%s' is not a valid Time."%value)
            
class TimeRadioGroupCol(StringCol):
    
    def __init__(self,  unique=False, default=None, options=None, description=None, treatEmptyStringAsNull=True):
        StringCol.__init__(self, notNULL=True, unique=unique, default=default, description=description, treatEmptyStringAsNull=treatEmptyStringAsNull)
        if options == None:
            raise SQLError('You must specify the allowed options for this field.')
        else: # XXX Should do proper error checking.
            self.sql_options = []
            for option in options:
                self.sql_options.append((option.asSQL(), option.asSQL()))
            
    def sql_buildField(self, name, value, description):
        return lemon.html.form.RadioGroup(name, options=self.sql_options, default=value, description=description)

    def sql_valid(self, value): # XXX Derive.
        "Field level validation."
        if type(value) == type(''):
            if not StringCol.sql_valid(self, value):
                return False
            else:
                try:
                    lemon.db.Time(sql=value)
                except lemon.db.DateError:
                    self.sql_setError('The date chosen is not valid.')
                    return False
                else:
                    return True
        else:
            if value.__class__.__name__ == 'Time':
                return True
            return False
    
    def builderValue(self):
        value = self.sql_field.get()
        if self.valid(value):
            return lemon.db.Time(sql=value)
        else:
            raise SQLError("'%s' is not a valid Time."%value)
            
            
#################################################################################################################
#
# Query Builder
#

class QueryFunctions:
    def NOW(self):
        return Query("NOW")
    def LIKE(self, pattern):
        return Query("LIKE " + " '" + pattern + "'")
    def BETWEEN(self, first, last):
        return Query("BETWEEN " + " '" + first + "' AND '" + last + "'")
        
func = QueryFunctions()

class QueryBuilder:
    def __init__(self, columns):
        self.__dict__['column'] = columns
        
    def __setattr__(self, name, value):
        raise SQLError('You can\'t set values.')
        
    def __getattr__(self, name):
        for col in self.column:
            if col.sql_getName() == name:
                return Query(name, type=col)
        raise SQLError("Column named '%s' not found."%name)
        
    def __repr__(self):
        fieldNames = []
        for col in self.column:
            fieldNames.append(col.sql_getName())
        return "<QueryBuilder instance: '%s'>"%(', '.join(fieldNames))
        
class Query:

    def __init__(self, name, list=[], type='mixed'):
        self.type = type
        self.name = name
        self.list = list
        self.param = '%s'
        self.operators =    {
                                '<':'<',
                                '<=':'<=',
                                '==':'=',
                                '<>':'<>',
                                '>':'>',
                                '>=':'>=',
                                '&':' AND ',
                                '|':' OR ',
                                '~':' NOT ',
                                'abs':'ABS',
                                '**':'POW',
                                '%':'MOD',
                                '+':'+',
                                '-':'-',
                                '*':'*',
                                '/':'/',
                            }
                            
    def merge(self,name,list):
        newList = []
        for l in list:
            if type(l) in[type(1), type(1.0004), type(1L)]:
                newList.append(str(l))
            else: # Assume string or can be converted to string type.
                newList.append("'"+str(l)+"'")
        return name%tuple(newList)
        
        # XXX NEW RULE... Any derived ColType must use a class constructor.. that way we know how to 
        # handle them in this constructor. eg Email('james@jimmyg.org') rather than 'james@jimmyg.org' which
        # would be handled like a string.
        
    def __str__(self):
        return self.merge(self.name, self.list)
        
    def __repr__(self):
        return "<Query instance: '%s'>"%self.merge(self.name, self.list)
    
    def checkTypes(self, left, right):
        if type(left) == InstanceType and left.__class__.__name__ == 'Query':
            # then left.type should be the column instance or a string
            try:
                left.type.sql_classID
            except AttributeError:
                raise TypeError("Column names should be Query() objects not '%s'."%col.__class__.__name__)
            else:
                if left.type.sql_classID == "THIS IS A James Gardner ColType Derived Class.":
                    if left.type.sql_valid(right):
                        return left.type.sql_encodeAsQuery(right)
                    else:
                        raise TypeError("'%s' is not a valid value for column '%s'."%(right, left.type.sql_getName()))
        raise TypeError("Column names should be Query() objects not '%s'."%type(left))
        
    def operator(self, left, right, operator):
        list = []
        
        leftString = ''
        rightString = ''
        
        for l in left.list:
            list.append(l)
        leftString = left.name

        if type(right) == InstanceType and right.__class__.__name__ == 'Query':
            for l in right.list:
                list.append(l)
            rightString = right.sql_getName()
        else:
            # check that right is type as left.type
            if self.checkTypes(left, right):
                list.append(right)
                rightString = self.param
            else:
                raise TypeError("'%s' columns should have values of type '%s' not '%s'."%(left.name, left.type, type(right)))
        output = '('+ leftString + ' ' + self.operators[operator] + ' ' + rightString +')'
        return Query(output, list)
        
    def logical(self, left, right, operator):
        list = []
        
        leftString = ''
        rightString = ''
        
        for l in left.list:
            list.append(l)
        leftString = left.name

        if type(right) is type(Query('',())): # XXX More checking reqired
            for l in right.list:
                list.append(l)
            rightString = right.name
        else:
            list.append(right)
            rightString = self.param
        
        output = '('+ leftString + ' ' + self.operators[operator] + ' ' + rightString +')'
        return Query(output, list)
        
    def prepend(self, right, operator):
        return Query('('+ self.operators[operator] + ' ' + str(right.name) +')',right.list) 

    #
    # Comparison Operators
    # <,<=,==,<>,>,>=
    #
    
    def __lt__(self, other):
        return self.operator(self, other, '<')
    def __le__(self, other):
        return self.operator(self, other, '<=')
    def __eq__(self, other):
        return self.operator(self, other, '==')
    def __ne__(self, other):
        return self.operator(self, other, '<>')
    def __gt__(self, other):
        return self.operator(self, other, '>')
    def __ge__(self, other):
        return self.operator(self, other, '>=')
    
    #
    # Logical Operators
    # &,|,~
    #
    
    def __and__(self, other):
        return self.logical(self, other, '&')
    def __or__(self, other):
        return self.logical(self, other, '|')
    def __invert__(self): # WARNING: This one is diferent
        return self.prepend(self, '~')

    #
    # Numeric Operators
    # +,-,*,/,**,abs(),%
    #
    
    def __add__(self, other):
        return self.operator(self, other, '+')
    def __sub__(self, other):
        return self.operator(self, other, '-')
    def __mul__(self, other):
        return self.operator(self, other, '*')
    def __div__(self, other):
        return self.operator(self, other, '/')
    def __pow__(self, other):
        return self.operator(self, other, '**')
    def __mod__(self, other):
        return self.operator(self, other, '%')


#################################################################################################################
#
# Table
#

class Table:
    "Base class for a database table object."

    #
    # Overridden Methods
    #
    
    def __repr__(self):
        return '<Table instance: %s>'%self.table
        
    def __init__(self, database=None):
        "Work the magic... Initialise the derived class by setting up the appropriate methods and classes."
        self.__dict__['column'] = []
        
        # Check a connection is provided.
        try:
            self.order
        except AttributeError:
            self.__dict__['order'] = []
            
        # Check a connection is provided.
        if not database:
            try:
                self.database
            except AttributeError:
                raise SQLError('No db database specified.')
        else:
            self.database = database
            
        # First time the driver is initialised.
        driver.database = self.database
        # Check a table name is provided.
        try:
            self.table
        except AttributeError:
            self.__dict__['table'] = self.__class__.__name__
        
        column = []
        # Add ColType specified in the derived class to the list self.ColTypeTypes
        for name, value in getmembers(self):
            if type(value) is InstanceType: # XXX Maybe some code testing base classes.
                if value.__dict__.has_key('sql_classID'):
                    if value.sql_classID == "THIS IS A James Gardner ColType Derived Class.":
                        upperName = name.upper()
                        if upperName in reservedWords:
                            raise SQLError("'%s' is a reserved word and cannot be used as a column name."%name)

                        value._init_(name, self.table, self.database)
                        column.append(value)
                        
        # Get the fields in the order specified by order.
        for k in self.order:
            matched = False
            for col in column[:]:
                if col.sql_getName() == k:
                    self.column.append(col)
                    column.pop(column.index(col))
                    matched = True
                    break
            if not matched:
                raise SQLError("The field '%s' doesn't exist. There may be an error in your table definition order list."%k)
        # Get the fields not matched
        for col in column:
            self.column.append(col)
                
    def __getitem__(self, name):
        return self.row(int(name))

    def __setitem__(self, name, value):
        raise SQLError('You cannot set a row like this. Use sql_setValue() instead.')
        
    #
    # Add/Remove Rows
    #
    
    def _newID(self):
        return driver.newID(self.table)
        
    def add(self, all=None, **params):
        "Add a new row to the table. Maps to the INSERT command. params contains the columnName=value pairs."
        
        if type(all) == type({}): # All is a reserved word so can't be a column so there should be no confusion.
            params = all
            
        # Generate a new ID
        rowid = self._newID()
        
        # Check that the params are valid
        names = []
        for col in self.column:
            names.append(col.sql_getName())
        for k in params.keys():
            if k not in names:
                raise SQLError("'%s' is not a column in the %s table."%(k,self.table))
                
        # Setup the variables
        fields = []
        values = []
        
        for col in self.column:
            # Add the field name
            fields.append(col.sql_getName())
            if col.sql_getName() in params.keys():
                # Validate and add any value for the field.
                if col.sql_valid(params[col.sql_getName()]):
                    values.append(col.sql_encode(params[col.sql_getName()]))
                else:
                    raise SQLError("'%s' is not a valid value for the column '%s'. %s"%(params[col.sql_getName()],col.sql_getName(),self._getCol(col.sql_getName()).sql_getError()))
            else:
                if col.sql_default:
                    # Add the defaults
                    values.append(col.sql_default)
                elif col.sql_notNULL:
                    raise SQLError("You must specify a '%s' field since it cannot be NULL."%col.sql_getName())
                else:
                    values.append(None)

        # Add the item to the database
        driver.addRow(self.table, fields, values)
        
        # Retrun the corresponding row object
        return self.row(rowid)
        
    def remove(self, rowid): # XXX Merge with below.
        "Remove the row with the rowid 'rowid' from the table."
        if self.rowExists(rowid):
            return  driver.removeRow(self.table, rowid)
        raise SQLError("The row with rowid '%s' doesn't exist."%rowid)

    def removeRows(self, query):
        "Maps to the DELETE object."
        return driver.removeRow(self.table, query)
    
    #
    # Get Rows
    #

    def row(self, rowid):
        "Return the Row object with the rowid 'rowid'."
        return Row(self.database, self.table, self.column, rowid)
            
    def emptyRow(self):
        return Row(self.database, self.table, self.column, None)


    def select(self, where=None, order=None):
        "Returns row objects for rows matching the criteria."
        #raise SQLError("There appears to be a major bug in this function. Don't use it.")
        rowids = self.rowids(where, order)
        
        res = []
        for rowid in rowids:
            #y = self.row(rowid)
            res.append(Row(self.database, self.table, self.column, None))

        #raise Exception(str(res))
        return res
            
    def query(self):
        return self.emptyRow().var
    #
    # Get IDs Function
    #
    
    def rowid(self, id):
        rowids = self.rowids(where='rowid='+str(id))
        if len(rowids) == 0:
            raise SQLError("No items matching your query.")
        if len(rowids) > 1:
            raise SQLError("More than one item matching your query.")
        return rowids[0]
        
    def rowids(self, where=None, order='rowid'):
        "Get a list of row rowids matching the criteria."
        return driver.rowids(self.table, where, order)

    #
    # SQL Funcitons
    #
    
    def max(self, name):
        "Returns the highest value in the column 'name'."
        return driver.max(self.table,name)
        
    def min(self, name):
        "Returns the lowest value in the column 'name'."
        return driver.min(self.table,name)
        
    def sum(self, name):
        "Returns the sum of the values in the column 'name'."
        return driver.sum(self.table,name)
        
    def average(self, name):
        "Returns the average value of the column 'name'."
        return driver.average(self.table,name)
        
    #
    # Functions testing existence
    #
    
    def exists(self):
        return driver.exists(self.table)

    def rowExists(self, rowid):
        if not rowid:
            raise SQLError("You haven't specified a row rowid to remove.")
        return driver.rowExists(self.table, rowid)
        
    def columnExists(self, name):
        for col in self.column:
            if col.sql_getName() == name:
                return True
        return False

    #
    # Create / Remove Tables
    #
    
    def create(self):
        "Create the table with the options specified."
        if self.database.tableExists(self.table):
            raise SQLError("The table '%s' already exists."%self.table)
        
        # (name, type, notNULL, default, auto, primary, unique)
        #fields = [('rowid', self.database.INTEGER, None, None, True, True, None)]
        fields = [('rowid', self.database.INTEGER, None, None, None, None, None)]
        for col in self.column:
            name = col.sql_getName()
            type = self.database.__dict__[col.sql_type()]
            notNULL = False
            if col.sql_notNULL == True:
                notNULL = True
            default = None
            if default == None:
                pass
            elif type(col.sql_default) == type(''): # Changed this from self.default.
                default = "'"+col.sql_default+"'"
            else:
                default = str(col.sql_default)
            unique = col.sql_unique
            #(name, type, notNULL, default, auto, primary, unique)
            if self.database.__class__.__name__ <> 'gadflyCursor':
                fields.append((name, type, notNULL, default, False, False, unique))
            else:
                fields.append((name, type))
        #self.database.autoExecute=False
        #raise TypeError(self.database.CREATE(self.table, tuple(fields)))
        #self.database.autoExecute=True
        self.database.CREATE(self.table, tuple(fields))

    def drop(self):
        "Drop (remove) the table from the database destroying all its contents."
        if self.exists():
            return self.database.DROP(self.table)
        raise SQLError("The table '%s' doesn't exist."%self.table)
        
    #
    # Private Methods
    #

    def _getCol(self, name):
        "Return the column instance for the column in the table with the name 'name'."
        for col in self.column:
            if col.sql_getName() == name:
                return col
        raise SQLError("There is no column named '%s'."%name)

#################################################################################################################
#
# Row
#

class Row:
    
    #
    # Row Initialisation
    #
    
    def __init__(self, database, table, column, rowid=None):
        """
        1. Creates a ColumnWrapper object so that the column list can be accessed via its attributes. 
           Attributes are then overridden so that it appears as the Row object is the column wrapper object. 
           This means that attribute access works for the columns and values needed from the row such as row.var which is the query builder.
        
        2. Actually retrieves the data from the database, decodes it and assigns it to the columns using sql_setValue()
        
        3. Assigns the current value to the HTML field representation using sql_setField()
        """
        self.__dict__['rowid'] = rowid
        self.__dict__['column'] = ColumnWrapper(column)
        self.__dict__['table'] = table
        self.__dict__['database'] = database
        self.__dict__['var'] = QueryBuilder(self.column)
        
       # print '<br><br>'+str(rowid) +'<br>' + repr(column)
        
        if self.rowid:
            # Prepare 
            fields = []
            for col in self.column.asList():
                fields.append(col.sql_getName())
                
            
            rows = driver.getRow(table, fields, self.rowid)
            if not rows:
                raise SQLError('There was an error retrieving the Data from the database.')
                
            
            # Array of ColType objects.  
            
            for i in range(len(self.column.asList())):
                self.column[i].sql_setValue(self.column[i].sql_decode(rows[0][i]))

        else:
            pass
            
        for col in self.column.asList():
            if col.sql_value:
                col.sql_setField(col.sql_value)
            elif col.sql_default: 
                col.sql_setField(col.sql_default)
            else:
                col.sql_setField(None)
            
    #
    # Transformation Methods
    #
            
    def form(self, name='form', action='', method='post', stickyData={}, enctype='multipart/form-data', submit='Submit'):
        """Generate a lemon.html.form.Form object for the row, using the values of the columns in the 
        particular row and the method attributes of this function."""
        a=self
        class Form(lemon.html.form.Form):
            
            def setup(self):
                import lemon.db
                # Add the columns in name order. Or you could add each field individually in the order you want them.
                for col in a.column.asList():
                    if not col.sql_hidden:
                        self.addField(col)
                    
                # The preffered way of adding submit buttons is as actions so Submit buttons are normally not used.
                self.addAction(submit)
                
        return Form(name, action, method, stickyData, enctype)
        
    def dict(self):
        # XXX Possible issue with dates?
        d = {}
        for col in self.column.asList():
            d[col.sql_name] = col.sql_value
        return d

    #
    # Representaion
    #
    
    def __repr__(self):
        fields = ''
        for col in self.column.asList():
            if type(col.sql_value) is type(''):
                fields += col.sql_getName() + '=\'' + col.sql_value +'\', '
            else:
                fields += col.sql_getName() + '=' + str(col.sql_value)+', '
        return "<Row rowid=%s: %s>"%(self.rowid, fields[:-2])
    
    #
    # Comparison Operators
    #
    
    def __lt__(self, other):
        raise SQLError('A row cannot be less than another row.')
    def __le__(self, other):
        raise SQLError('A row cannot be less than another row.')
    def __gt__(self, other):
        raise SQLError('A row cannot be greater than another row.')
    def __ge__(self, other):
        raise SQLError('A row cannot be greater than another row.')
        
    def __eq__(self, other): 
        if other == None:
            return 0
        # check types
        if type(other) is InstanceType:
            if other.__class__.__name__ == self.__class__.__name__:
                # Check lenghts
                if len(other) == len(self):
                    for i in range(len(self.column)):
                        # Check type of ColType
                        if other.column[i].__class__.__name__ <> self.column[i].__class__.__name__:
                            return 0
                        # Check values
                        if other.column[i].sql_getValue() <> self.column[i].sql_getValue():
                            return 0
                    return 1 # Same number of columns, type of columns, values => must be the same??
                else:
                    return 0
        else:
            return 0
            #raise SQLError('You can only compare %s objects with other %s objects, not %s objects.'%(self.__class__.__name__,self.__class__.__name__,other.__class__.__name__))

    def __ne__(self, other): 
        return not self.__eq__(other)
        
    def __nonzero__(self):
        if len(self.column) > 0:
            return True
        return False
        
    def __len__(self):
        return len(self.column)

    #def __contains__(self, item)

    #
    # Core Row methods
    #
    
    def _getCol(self, name):
        "Finds a column by name or by order in the list. Returns that column object."
        if type(name) is type(1):
            return self.column[name]
        elif type(name) is type(''):
            for col in self.column.asList():
                if col.sql_getName() == name:
                    return col
            raise SQLError("There is no column named '%s'."%name)
        else:
            raise SQLError("Colmuns can only be indexed as a string or an integer.")
            
    def add(self, **params):
        "ERROR: You can't add a row to a row. Instead add a row to the table object."
        raise SQLError("You can't add a row to a row. Instead add a row to the table object.")  
        
    def set(self, all=None, **params):
        "UPDATE Multiple rows"
        
        if type(all) is type({}):
            params = all
        
        
        # Setup the variables
        fields = ['rowid']
        values = [int(self.rowid)]
        
        # Check for Nones that shouldn't be there.
        for k,v in params.items():
            if v == None:
                if self._getCol(k).sql_notNULL == True:
                    raise SQLError("The field '%s' cannot be set to None since notNULL has been set True."%k)

        # Fill the fields
        i = 0
        for k, v in params.items():
            fields.append(k)
            if self._getCol(k).sql_valid(v):
                values.append(self._getCol(k).sql_encode(v))
            else:
                raise SQLError("'%s' is not a valid value for the column '%s'."%(v,f))
                
        # Add the item to the database
        #if mode=='add':
        if driver.rowExists(self.table, self.rowid):
            driver.updateRow(self.table, self.rowid, fields[1:], values[1:])
        else:
            driver.addRow(self.table, fields, values)
            
        #else:
        #    driver.updateRow(self.table, fields, values)

        for f in range(1,len(fields)):
            self._getCol(fields[f]).sql_setValue(values[f])
            
        #self.database.commit()
        #raise SQLError("This function doesn't update the core values. XXX TODO")
    #~ def getDB(self, name):
        #~ "Return a ColType object."
        
    def __getattr__(self, name):
        return self._getCol(name).sql_getValue() # WARNING: Reads the value.. doesn't get it from the DB.
            
    def __setattr__(self, name, value):
        self._getCol(name).sql_setDB(value)
        
    def __getitem__(self, name):
        return self._getCol(name).sql_getValue() # WARNING: Reads the value.. doesn't get it from the DB.
            
    def __setitem__(self, name, value):
        self._getCol(name).sql_setDB(value)


#################################################################################################################
#
# Column
#

class ColumnWrapper:
    
    def __str__(self):
        c = []
        for col in self.column:
            c.append(col.sql_getName())

        return '['+', '.join(c)+']'
        
    def __repr__(self):
        return '<Column instance acting as a list of ColType instances: '+self.__str__()[1:-1]+' >'
        
    def __init__(self, column):
        self.__dict__['column'] = []
        for c in column:
            self.column.append(c)
    
    
    def _getCol(self, name, warn=True):
        if type(name) is type(1):
            return self.column[name]
        elif type(name) is type(''):
            for col in self.column:
                if col.sql_getName() == name:
                    return col
            if warn:
                raise SQLError("There is no column named '%s'."%name)
        else:
            if warn:
                raise SQLError("Colmuns can only be indexed as a string or an integer.")
            
    def __getattr__(self, name):
        return self._getCol(name)
        
    def asList(self):
        return self.column
    
    def __setattr__(self, name, value):
        raise SQLError("Cannot set a column like this.")
        
    def __getitem__(self, name):
        return self._getCol(name)
            
    def __setitem__(self, name, value):
        raise SQLError("Cannot set a column like this.")
        
    def __len__(self):
        return len(self.column)

    #def __contains__(self, name):
        
        #return self._getCol(self, name)
    
