"""Lemon DB interface to the odbc cursor.

Version:

    File Version 0.1 for LEMON 0.2
    
Description:
    
    Module containing the odbc cursor.
"""

__author__ = "James Gardner [email: james at xecos.com]"
__date__ = "02 July 2003"
__version__ = "Version 0.1 for LEMON 0.2"[-3:]
__credits__ = "Guido van Rossum, for an excellent programming language."



import os
if os.name == 'nt' or os.name == 'dos':
    import mx.ODBC.Windows as odbc
elif os.name == 'posix':
    raise lemon.db.DBError('ODBC driver not tested for posix.')
    import mx.ODBC.iODBC as odbc
else:
    raise lemon.db.DBError('ODBC driver not implemented for your platform')
    
import base
import lemon.db


class odbcConnection(base.baseConnection):
    def __init__(self, database):
        'Set up the database connection'
        self.connection = odbc.connect(database) 
        
    def cursor(self, autoExecute=True, fetchMode='tuple', autoConvert=False, convertMode='table', colTypesName='ColTypes', types={}):
        "Return our custom cursor rather than the default connection cursor."
        return odbcCursor(self.connection.cursor(), autoExecute, fetchMode, autoConvert, convertMode, colTypesName, types)

    
class odbcCursor(base.baseCursor):

    def setupOptions(self):
        return [
                    [True, False],              # autoExecute
                    ['tuple','dict','dtuple'],  # fetchMode
                    [True, False],              # autoConvert
                    ['table'],                  # convertMode
                ]
                
                
    # XXX These were uncommented.. haven't been tested yet.
    
    def decodeDate(self, field):
        return base.baseCursor.decodeDate(self, str(field)[0:10])
        
    def decodeTime(self, field):
        return base.baseCursor.decodeTime(self, str(field)[11:19])

    def decodeDateTime(self, field):
        return base.baseCursor.decodeDateTime(self, str(field)[:-3])
        
    def setupFields(self):
        
        self.CHAR       = (1,)
        self.STRING     = (2,)
        self.BINARY     = (3,)
        self.INTEGER    = (4,)
        self.FLOAT      = (5,)
        self.DATE       = (6,)
        self.TIME       = (7,)
        self.DATETIME   = (8,)

        self.fieldTypes =   {
                                self.CHAR       :"CHAR",
                                self.STRING     :"VARCHAR",
                                self.BINARY     :"MEMO", # XXX not sure this will work for other ODBC interfaces.
                                self.INTEGER    :"SMALLINT",
                                self.FLOAT      :"FLOAT",
                                self.DATE       :"DATE",
                                self.TIME       :"TIME",
                                self.DATETIME   :"DATETIME",
                               
                            }
