"""Implementation of a database storage driver for the session module."""

import lemon, pickle, StringIO, lemon, lemon.session, time
import base

class File(base.Base):
    """The main session class for file-based storage of sessions.
    See the information in lemon.session.start() for details of how to use it. The class should not be called 
    directly becuase error-checking occurs in lemon.session.start()."""
        
    def _addSession(self, sessionID):
        "Create a new session setting the 'Created' and 'Accessed' variables."
        self.params['database'].INSERT(   
                                        lemon.config.get('session','db.table'), 
                                        (lemon.config.get('session','db.sessionID'), lemon.config.get('session','db.app'), lemon.config.get('session','db.itemKey'),lemon.config.get('session','db.itemValue')),
                                        (self.params['sessionID'],lemon.config.get('session','db.table'),'Created',int(time.time())),
                                    )
        self.params['database'].INSERT(   
                                        lemon.config.get('session','db.table'), 
                                        (lemon.config.get('session','db.sessionID'), lemon.config.get('session','db.app'), lemon.config.get('session','db.itemKey'),lemon.config.get('session','db.itemValue')),
                                        (self.params['sessionID'],lemon.config.get('session','db.table'),'Accessed',int(time.time())),
                                    )
        self.params['database'].commit()
                                    
    def _removeSession(self, sessionID):
        "Remove session information from the session store."
        self.params['database'].DELETE(lemon.config.get('session','db.table'), WHERE="SessionID='"+sessionID+"'")
        self.params['database'].commit()
        
    def _set(self, key, value, app=None, pickleValue=True):
        "Sets the variable 'key' to the value 'value' after pickling it. # XXX The app functionality is not tested"
        if not app:
            app = self.params['name']
        if pickleValue:
            out = StringIO.StringIO()
            pickle.dump(value,out)
            value = out.getvalue()
    
        rows = self.params['database'].SELECT(lemon.config.get('session','db.itemValue'),lemon.config.get('session','db.table'),WHERE="SessionID='"+self.params['sessionID']+"' and ItemKey='"+key+"'")
        if len(rows) == 0:
            self.params['database'].INSERT(lemon.config.get('session','db.table'), (lemon.config.get('session','db.sessionID'), lemon.config.get('session','db.app'),lemon.config.get('session','db.itemKey'), lemon.config.get('session','db.itemValue')), (self.params['sessionID'], self.params['name'], key, value))
            self.params['database'].commit()
        else:
            self.params['database'].UPDATE(lemon.config.get('session','db.table'), (lemon.config.get('session','db.itemValue'),), (value,), WHERE="SessionID='"+self.params['sessionID']+"' and ItemKey='"+key+"' and App='"+app+"'")  
            self.params['database'].commit()
    
    def _get(self, key, app=None, pickleValue=True):
        "Gets the variable 'key' from the session store."
        if not app:
            app = self.params['name']
        rows=self.params['database'].SELECT((lemon.config.get('session','db.itemValue')),FROM=lemon.config.get('session','db.table'),WHERE="SessionID='"+self.params['sessionID']+"' and ItemKey='"+key+"' and App='"+app+"'")  
        if not rows:
            return None
        else:
            if self.params['database'].fetchMode in ['dict','dtuple']:
                read = StringIO.StringIO(rows[0][lemon.config.get('session','db.itemValue')])
            elif 'tuple' == self.params['database'].fetchMode:
                read = StringIO.StringIO(rows[0][0])
            else:
                raise lemon.db.SessionError("Database fetchmode '%s' not understood."%self.params['database'].getFetchMode())
            if pickleValue:
                return pickle.load(read)
            else:
                return read
                
    def _delete(self, key):
        "Removes the variable from the session store."
        self.params['database'].DELETE(lemon.config.get('session','db.table'),WHERE="SessionID='"+self.params['sessionID']+"' and ItemKey='"+key+"'")  
        self.params['database'].commit()
        
    def _cleanupSessions(self):
        "Remove all expired sessions."
        rows = self.params['database'].SELECT('SessionID',lemon.config.get('session','db.table'), WHERE="ItemKey='SessionCreated' and ItemValue < "+str(int(time.time()) - int(self.params['expires'])))      
        for row in rows:
            self.params['database'].DELETE(lemon.config.get('session','db.table'), WHERE="SessionID='"+row[0]+"'")      
        self.params['database'].commit()
        
    def _updateAccessed(self):
        "Update the 'Accessed' variable of the session."
        self.params['database'].UPDATE(   
                                        lemon.config.get('session','db.table'), 
                                        lemon.config.get('session','db.itemValue'),
                                        int(time.time()),
                                        WHERE="SessionID='"+self.params['sessionID']+"' and App='Sessions' and ItemKey='Accessed'",
                                    )
        self.params['database'].commit()
                                    
    def _sessionExists(self, sessionID):
        "Returns True if the session already exists, False otherwise."
        rows = self.params['database'].SELECT(lemon.config.get('session','db.sessionID'),FROM=lemon.config.get('session','db.table'), WHERE="SessionID='"+sessionID+"'" )
        if rows:
            return True
        return False
        
    def _setupParams(self):
        "Initialise and error check and setup params."        
        # Check name
        if not self.params.has_key('database'):
            raise lemon.session.SessionError('You must have a valid db database to connect to a db session driver.')
            
        if not self.params.has_key('name'):
            raise lemon.session.SessionError('You must have a valid name to connect to a db session driver.')
        
        if len(self.params['name']) > 255:
            raise lemon.session.SessionError('The maximum length for an application name is 255 characters.')
