"""PHP-style authorisation functions.

Version:

    File Version 0.1 for LEMON 0.2
    
Usage:

    session = lemon.session.SessionDB(self.database, appName)  # Retrieve the session or create a new one
    auth    = lemon.auth.AuthDB(session)                # Create auth object from the session information
    
    if auth.isValid():                                  # See if the User is logged on else present login form.
        user = a.getUser()                              # Return a user object containing user's details.
        user.Password                                   # Access the data via attributes or dict style.
        user.getStatus()                                # Retrieve the entry for the appName column.

ToDO:

    addUser, RemoveUser etc.
    gadfly
    create the database if not already there?? 
    definately need config file similar to php.ini
"""

__author__ = "James Gardner [email: james at xecos.com]"
__date__ = "02 July 2003"
__version__ = "Version 0.1 for LEMON 0.2"[-3:]
__credits__ = "Guido van Rossum, for an excellent programming language."

import time, md5, lemon
import base

class DB(base.Base):
    
    def __init__(self, conn, table='Session'):
        #self.conn = conn
        #self.database = conn.cursor(autoConvert=True)
        self.database = conn
        self.table = table
        
    def createTables(self, autoCommit=False): # autoCommit needed for the Gadfly types information to be added.
        """Create a session table cable of holding session information from the default values in the Lemon.ini file."""
        errors = []
        if not self.database.tableExists(self.table+'Store'):
            self.database.create(
                    self.table+'Store',
                    (    
                        ('SessionID',   self.database.STRING),
                        ('App',         self.database.STRING),
                        ('ItemKey',     self.database.STRING),
                        ('ItemValue',   self.database.BINARY),
                    )
                )
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append("The '"+self.table+'Store'+"' table already exists.")

            
        if not self.database.tableExists(self.table+'s'):
            self.database.create(
                    self.table+'s',
                    (    
                        ('SessionID',   self.database.STRING),
                        ('Expires',     self.database.INTEGER),
                    )
                )
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append("The '"+self.table+'s'+"' table already exists.")
        return errors
        
    def dropTables(self, autoCommit=False):
        errors = []
        if self.database.tableExists(self.table+'Store'):
            self.database.drop(self.table+'Store')
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append("The "+self.table+'Store'+" table doesn't exist.")
        if self.database.tableExists(self.table+'s'):
            self.database.drop(self.table+'s')
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append("The "+self.table+'s'+" table doesn't exist.")
        return errors

    def tablesExist(self):
        if self.database.tableExists(self.table+'Store') and self.database.tableExists(self.table+'s'):
            return True
        return False